package com.vctgo.system.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.vctgo.common.core.constant.Constants;
import com.vctgo.common.core.constant.HttpStatus;
import com.vctgo.common.core.constant.UserConstants;
import com.vctgo.common.core.domain.R;
import com.vctgo.common.core.utils.StrUtils;
import com.vctgo.common.security.utils.SecurityUtils;
import com.vctgo.system.api.RemoteProfileService;
import com.vctgo.system.api.domain.SysPlatformRole;
import com.vctgo.system.api.domain.SysPlatformUser;
import com.vctgo.system.api.domain.dto.ProfileUserInfoDto;
import com.vctgo.system.domain.SysPlatformMenu;
import com.vctgo.system.domain.SysSimpleMenu;
import com.vctgo.system.domain.vo.MetaVo;
import com.vctgo.system.domain.vo.RouterVo;
import com.vctgo.system.domain.vo.TreeSelect;
import com.vctgo.system.mapper.SysPlatformMenuMapper;
import com.vctgo.system.mapper.SysPlatformRoleMapper;
import com.vctgo.system.mapper.SysPlatformRoleMenuMapper;
import com.vctgo.system.service.ISysPlatformMenuService;
import com.vctgo.system.util.ConstantUtils;
import com.vctgo.system.util.SysPlatformMenuTreeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 菜单 业务层处理
 *
 * @author vctgo
 */
@Service
public class SysPlatformMenuServiceImpl implements ISysPlatformMenuService {
    public static final String PREMISSION_STRING = "perms[\"{0}\"]";
    private static final Logger log = LoggerFactory.getLogger(SysPlatformMenuServiceImpl.class);
    @Autowired
    private SysPlatformMenuMapper menuMapper;

    @Autowired
    private SysPlatformRoleMapper roleMapper;

    @Autowired
    private SysPlatformRoleMenuMapper roleMenuMapper;

    @Autowired
    private RemoteProfileService remoteProfileService;

    /**
     * 根据用户查询系统菜单列表
     *
     * @param userId 用户ID
     * @return 菜单列表
     */
    @Override
    public List<SysPlatformMenu> selectMenuList(Long userId) {
        return selectMenuList(new SysPlatformMenu(), userId);
    }

    /**
     * 查询系统菜单列表
     *
     * @param menu 菜单信息
     * @return 菜单列表
     */
    @Override
    public List<SysPlatformMenu> selectMenuList(SysPlatformMenu menu, Long userId) {
        List<SysPlatformMenu> menuList = null;
        // 管理员显示所有菜单信息
        if (SysPlatformUser.isAdmin(userId)) {
            menuList = menuMapper.selectMenuList(menu);
        } else {
            menu.getParams().put("userId", userId);
            menuList = menuMapper.selectMenuListByUserId(menu);
        }
        return menuList;
    }

    /**
     * 查询所有开启状态菜单精简信息
     *
     * @return 菜单列表
     */
    @Override
    public List<SysSimpleMenu> selectSimpleMenuList() {
        return menuMapper.selectSimpleMenuList();
    }

    /**
     * 根据用户ID查询权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectMenuPermsByUserId(Long userId) {
        List<String> perms = menuMapper.selectMenuPermsByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (String perm : perms) {
            if (StrUtils.isNotEmpty(perm)) {
                permsSet.addAll(Arrays.asList(perm.trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 根据角色ID查询权限
     *
     * @param roleId 角色ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectMenuPermsByRoleId(Long roleId) {
        List<String> perms = menuMapper.selectMenuPermsByRoleId(roleId);
        Set<String> permsSet = new HashSet<>();
        for (String perm : perms) {
            if (StrUtils.isNotEmpty(perm)) {
                permsSet.addAll(Arrays.asList(perm.trim().split(",")));
            }
        }
        return permsSet;
    }


    /**
     * 根据用户ID查询菜单
     *
     * @param userId 用户名称
     * @return 菜单列表
     */
    @Override
    public List<SysPlatformMenu> selectMenuTreeByUserId(Long userId) {
        List<SysPlatformMenu> platformMenuList = null;
        if (SecurityUtils.isAdmin(userId)) {
            platformMenuList = menuMapper.selectMenuTreeAll();
        } else {
            platformMenuList = menuMapper.selectMenuTreeByUserId(userId);

            //内部资源管理使用---->获取路由信息
            getProfileMenuAuth(platformMenuList, userId);
        }
        platformMenuList = SysPlatformMenuTreeUtils.getSysPlatformMenuTreeList(platformMenuList, ConstantUtils.PLAT_FORM_MENU_PARENT_ID_0);
        //System.out.println("获取路由信息："  + JSONArray.toJSON(platformMenuList));
        return platformMenuList;
    }

    /**
     * 根据角色ID查询菜单树信息
     *
     * @param roleId 角色ID
     * @return 选中菜单列表
     */
    @Override
    public List<Long> selectMenuListByRoleId(Long roleId) {
        SysPlatformRole role = roleMapper.selectRoleById(roleId);
        return menuMapper.selectMenuListByRoleId(roleId, role.isMenuCheckStrictly());
    }

    /**
     * 构建前端路由所需要的菜单
     *
     * @param menus 菜单列表
     * @return 路由列表
     */
    @Override
    public List<RouterVo> buildMenus(List<SysPlatformMenu> menus) {
        List<RouterVo> routers = new LinkedList<RouterVo>();
        for (SysPlatformMenu menu : menus) {
            RouterVo router = new RouterVo();
            router.setHidden("1".equals(menu.getVisible()));
            router.setName(getRouteName(menu));
            router.setPath(getRouterPath(menu));
            router.setComponent(getComponent(menu));
            router.setQuery(menu.getQuery());
            router.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), StrUtils.equals("1", menu.getIsCache()), menu.getPath()));
            List<SysPlatformMenu> cMenus = menu.getChildren();
            if (!cMenus.isEmpty() && cMenus.size() > 0 && UserConstants.TYPE_DIR.equals(menu.getMenuType())) {
                router.setAlwaysShow(true);
                router.setRedirect("noRedirect");
                router.setChildren(buildMenus(cMenus));
            } else if (isMenuFrame(menu)) {
                router.setMeta(null);
                List<RouterVo> childrenList = new ArrayList<RouterVo>();
                RouterVo children = new RouterVo();
                children.setPath(menu.getPath());
                children.setComponent(menu.getComponent());
                children.setName(StrUtils.capitalize(menu.getPath()));
                children.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), StrUtils.equals("1", menu.getIsCache()), menu.getPath()));
                children.setQuery(menu.getQuery());
                childrenList.add(children);
                router.setChildren(childrenList);
            } else if (menu.getParentId().intValue() == 0 && isInnerLink(menu)) {
                router.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon()));
                router.setPath("/inner");
                List<RouterVo> childrenList = new ArrayList<RouterVo>();
                RouterVo children = new RouterVo();
                String routerPath = innerLinkReplaceEach(menu.getPath());
                children.setPath(routerPath);
                children.setComponent(UserConstants.INNER_LINK);
                children.setName(StrUtils.capitalize(routerPath));
                children.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), menu.getPath()));
                childrenList.add(children);
                router.setChildren(childrenList);
            }
            routers.add(router);
        }
        return routers;
    }

    /**
     * 构建前端所需要树结构
     *
     * @param menus 菜单列表
     * @return 树结构列表
     */
    @Override
    public List<SysPlatformMenu> buildMenuTree(List<SysPlatformMenu> menus) {
        List<SysPlatformMenu> returnList = new ArrayList<SysPlatformMenu>();
        List<Long> tempList = menus.stream().map(SysPlatformMenu::getMenuId).collect(Collectors.toList());
        for (Iterator<SysPlatformMenu> iterator = menus.iterator(); iterator.hasNext(); ) {
            SysPlatformMenu menu = (SysPlatformMenu) iterator.next();
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(menu.getParentId())) {
                recursionFn(menus, menu);
                returnList.add(menu);
            }
        }
        if (returnList.isEmpty()) {
            returnList = menus;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param menus 菜单列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildMenuTreeSelect(List<SysPlatformMenu> menus) {
        List<SysPlatformMenu> menuTrees = buildMenuTree(menus);
        return menuTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据菜单ID查询信息
     *
     * @param menuId 菜单ID
     * @return 菜单信息
     */
    @Override
    public SysPlatformMenu selectMenuById(Long menuId) {
        return menuMapper.selectMenuById(menuId);
    }

    /**
     * 是否存在菜单子节点
     *
     * @param menuId 菜单ID
     * @return 结果
     */
    @Override
    public boolean hasChildByMenuId(Long menuId) {
        int result = menuMapper.hasChildByMenuId(menuId);
        return result > 0;
    }

    /**
     * 查询菜单使用数量
     *
     * @param menuId 菜单ID
     * @return 结果
     */
    @Override
    public boolean checkMenuExistRole(Long menuId) {
        int result = roleMenuMapper.checkMenuExistRole(menuId);
        return result > 0;
    }

    /**
     * 新增保存菜单信息
     *
     * @param menu 菜单信息
     * @return 结果
     */
    @Override
    public int insertMenu(SysPlatformMenu menu) {
        return menuMapper.insertMenu(menu);
    }

    /**
     * 修改保存菜单信息
     *
     * @param menu 菜单信息
     * @return 结果
     */
    @Override
    public int updateMenu(SysPlatformMenu menu) {
        return menuMapper.updateMenu(menu);
    }

    /**
     * 删除菜单管理信息
     *
     * @param menuId 菜单ID
     * @return 结果
     */
    @Override
    public int deleteMenuById(Long menuId) {
        return menuMapper.deleteMenuById(menuId);
    }

    /**
     * 校验菜单名称是否唯一
     *
     * @param menu 菜单信息
     * @return 结果
     */
    @Override
    public boolean checkMenuNameUnique(SysPlatformMenu menu) {
        Long menuId = StrUtils.isNull(menu.getMenuId()) ? -1L : menu.getMenuId();
        SysPlatformMenu info = menuMapper.checkMenuNameUnique(menu.getMenuName(), menu.getParentId());
        if (StrUtils.isNotNull(info) && info.getMenuId().longValue() != menuId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 获取路由名称
     *
     * @param menu 菜单信息
     * @return 路由名称
     */
    public String getRouteName(SysPlatformMenu menu) {
        String routerName = StrUtils.capitalize(menu.getPath());
        // 非外链并且是一级目录（类型为目录）
        if (isMenuFrame(menu)) {
            routerName = StrUtils.EMPTY;
        }
        return routerName;
    }

    /**
     * 获取路由地址
     *
     * @param menu 菜单信息
     * @return 路由地址
     */
    public String getRouterPath(SysPlatformMenu menu) {
        String routerPath = menu.getPath();
        // 内链打开外网方式
        if (menu.getParentId().intValue() != 0 && isInnerLink(menu)) {
            routerPath = innerLinkReplaceEach(routerPath);
        }
        // 非外链并且是一级目录（类型为目录）
        if (0 == menu.getParentId().intValue() && UserConstants.TYPE_DIR.equals(menu.getMenuType())
                && UserConstants.NO_FRAME.equals(menu.getIsFrame())) {
            routerPath = "/" + menu.getPath();
        }
        // 非外链并且是一级目录（类型为菜单）
        else if (isMenuFrame(menu)) {
            routerPath = "/";
        }
        return routerPath;
    }

    /**
     * 获取组件信息
     *
     * @param menu 菜单信息
     * @return 组件信息
     */
    public String getComponent(SysPlatformMenu menu) {
        String component = UserConstants.LAYOUT;
        if (StrUtils.isNotEmpty(menu.getComponent()) && !isMenuFrame(menu)) {
            component = menu.getComponent();
        } else if (StrUtils.isEmpty(menu.getComponent()) && menu.getParentId().intValue() != 0 && isInnerLink(menu)) {
            component = UserConstants.INNER_LINK;
        } else if (StrUtils.isEmpty(menu.getComponent()) && isParentView(menu)) {
            component = UserConstants.PARENT_VIEW;
        }
        return component;
    }

    /**
     * 是否为菜单内部跳转
     *
     * @param menu 菜单信息
     * @return 结果
     */
    public boolean isMenuFrame(SysPlatformMenu menu) {
        return menu.getParentId().intValue() == 0 && UserConstants.TYPE_MENU.equals(menu.getMenuType())
                && menu.getIsFrame().equals(UserConstants.NO_FRAME);
    }

    /**
     * 是否为内链组件
     *
     * @param menu 菜单信息
     * @return 结果
     */
    public boolean isInnerLink(SysPlatformMenu menu) {
        return menu.getIsFrame().equals(UserConstants.NO_FRAME) && StrUtils.ishttp(menu.getPath());
    }

    /**
     * 是否为parent_view组件
     *
     * @param menu 菜单信息
     * @return 结果
     */
    public boolean isParentView(SysPlatformMenu menu) {
        return menu.getParentId().intValue() != 0 && UserConstants.TYPE_DIR.equals(menu.getMenuType());
    }

    /**
     * 根据父节点的ID获取所有子节点
     *
     * @param list     分类表
     * @param parentId 传入的父节点ID
     * @return String
     */
    public List<SysPlatformMenu> getChildPerms(List<SysPlatformMenu> list, int parentId) {
        List<SysPlatformMenu> returnList = new ArrayList<SysPlatformMenu>();
        for (Iterator<SysPlatformMenu> iterator = list.iterator(); iterator.hasNext(); ) {
            SysPlatformMenu t = (SysPlatformMenu) iterator.next();
            // 一、根据传入的某个父节点ID,遍历该父节点的所有子节点
            if (t.getParentId() == parentId) {
                recursionFn(list, t);
                returnList.add(t);
            }
        }
        return returnList;
    }

    /**
     * 递归列表
     *
     * @param list
     * @param t
     */
    private void recursionFn(List<SysPlatformMenu> list, SysPlatformMenu t) {
        // 得到子节点列表
        List<SysPlatformMenu> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysPlatformMenu tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysPlatformMenu> getChildList(List<SysPlatformMenu> list, SysPlatformMenu t) {
        List<SysPlatformMenu> tlist = new ArrayList<>();
        Iterator<SysPlatformMenu> it = list.iterator();
        while (it.hasNext()) {
            SysPlatformMenu n = (SysPlatformMenu) it.next();
            if (n.getParentId().longValue() == t.getMenuId().longValue()) {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysPlatformMenu> list, SysPlatformMenu t) {
        return getChildList(list, t).size() > 0;
    }

    /**
     * 内链域名特殊字符替换
     *
     * @return
     */
    public String innerLinkReplaceEach(String path) {
        return StrUtils.replaceEach(path, new String[]{Constants.HTTP, Constants.HTTPS, Constants.WWW, "."},
                new String[]{"", "", "", "/"});
    }

    private void getProfileMenuAuth(List<SysPlatformMenu> platformMenuList,Long userId){
        try {
            //剔除角色关联的特定菜单
            List<String> profileMenuAuthList = Arrays.asList(
                    ConstantUtils.PROFILE_USER_AUTH_WHOLE,                                  //综合查询
                    ConstantUtils.PROFILE_USER_AUTH_CONTROL,                                //园所资料
                    ConstantUtils.PROFILE_USER_AUTH_ACHIEVEMENT,                            //方案成果
                    ConstantUtils.PROFILE_USER_AUTH_SCHEME_TEMPLATE,                        //方案模板
                    ConstantUtils.PROFILE_POLICY_AUTH,                                      //政策文件
                    ConstantUtils.PROFILE_PLACE_RESOURCE_AUTH,                              //地方资源
                    ConstantUtils.PROFILE_THEORY_AUTH,                                      //专业理论
                    ConstantUtils.PROFILE_USER_AUTH_CASE,                                   //活动案例
                    ConstantUtils.PROFILE_USER_AUTH_DIRECTION,                              //主题方向
                    ConstantUtils.PROFILE_USER_AUTH_STANDARD,                               //标准管理
                    ConstantUtils.PROFILE_USER_AUTH_LABEL,                                  //标签管理
                    ConstantUtils.PROFILE_USER_AUTH_AUTH,                                   //权限配置
                    ConstantUtils.PROFILE_USER_AUTH_KINDER,                                 //园所资料授权
                    ConstantUtils.PROFILE_USER_GAME_AUTH                                    //区域游戏授权
            );
            platformMenuList.removeIf(sysPlatformMenu -> profileMenuAuthList.contains(sysPlatformMenu.getPerms()));
            //添加内部资源管理用户获取菜单权限，如果为“启用”状态且“时间”有效
            R<ProfileUserInfoDto> profileUserInfoDtoR = remoteProfileService.getValidUserInfoByUserId(userId);
            if (HttpStatus.SUCCESS == profileUserInfoDtoR.getCode()) {
                if (Objects.nonNull(profileUserInfoDtoR.getData()) && profileUserInfoDtoR.getData().getAccountStatus().equals(ConstantUtils.PROFILE_USER_ACCOUNT_STATUS_10)) {
                    ProfileUserInfoDto profileUserInfoDto = profileUserInfoDtoR.getData();
                    if (StrUtils.isNotEmpty(profileUserInfoDto.getMenuAuthCode())) {
                        List<String> menuAuthCodeList = new ArrayList<>(Arrays.asList(profileUserInfoDto.getMenuAuthCode().split(",")));
                        menuAuthCodeList.remove("");
                        Set<String> menuAuthCodeSet = new HashSet<>(menuAuthCodeList);
                        //获取菜单权限字符对应的菜单路由
                        List<SysPlatformMenu> sysPlatformMenuList = menuMapper.selectList(new QueryWrapper<SysPlatformMenu>().lambda().in(SysPlatformMenu::getPerms, menuAuthCodeSet));
                        platformMenuList.addAll(sysPlatformMenuList);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取内部资源管理用户权限失败:{}", e.getMessage());
        }
    }
}
